#!/bin/bash
# SCENARI app installer script
# LICENCE[[
# Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.0
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is kelis.fr code.
#
# The Initial Developer of the Original Code is
# samuel.monsarrat@kelis.fr
#
# Portions created by the Initial Developer are Copyright (C) 2006-2016
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
#
# Alternatively, the contents of this file may be used under the terms of
# either of the GNU General Public License Version 2 or later (the "GPL"),
# or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
# in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
# instead of those above. If you wish to allow use of your version of this file
# only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
# others to use your version of this file under the terms of the MPL, indicate
# your decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
# delete the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
# ]]LICENCE

# Init variables
vMyName=`basename $0`
appName="SCENARIchain"
appNameFull="SCENARIchain 4.2"
appVersion="4.2"
appVersionFull="4.2.101"
appPublisher="SCENARI"
appClientName="scApp"
appDescription="Créer des contenus et publier des documents dans le cadre d'une chaîne éditoriale créée avec SCENARIbuilder."
appPrefixDefaultRoot="/opt"
linkPrefixDefaultRoot="/usr"
appPrefixDefaultUser=$HOME
linkPrefixDefaultUser=$HOME
logFile="/tmp/"$appName$appVersionFull"-install_"$(date +%Y.%m.%d-%H.%M.%S)".log"
tmpDir="/tmp/scinstall"
ecoLine=""
forceMode="no"
appPath=""
locateExists=""
incorrectArch="false"

# Start...
if echo "$1" | grep -q h ; then
	echo
	echo "$appNameFull installer"
	echo
	echo "This script will install $appNameFull on your computer."
	echo "You can either run this script as root, in which case a system-wide" 
	echo "installation will be made and a menu entry can be added, or you can run"
  echo "this script as a normal user in which case $appNameFull will be installed"
  echo "in your home directory."
	echo
  echo "This installer has the following options:"
  echo "-h   prints this message and exits"
  echo "-f   'force' mode, the installer does not interact with the user, all"
  echo "     default choices are used. If the application is already installed"
  echo "     it will be replaced. This is basically equivalent to pressing return"
  echo "     for each question the the installer normally asks the user."
	echo
  echo "This installer script is Copyright (C) 2006-2016 samuel.monsarrat@kelis.fr"
  echo "Licensed under MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.0"
	echo
	exit 0
fi
if echo "$1" | grep -q f ; then
	forceMode="yes"
fi
if echo "$1" | grep -q l ; then
	lastScAppBinPathFile="$2"
else
	lastScAppBinPathFile=$tmpDir"/lastScAppBinPath.txt"
fi

locate --version >> /dev/null 2>&1
if [ "$?" -ne "0" ];then
	locateExists="no"
else
	locateExists="yes"
fi

if [ ! -f "$appName.tar.gz" ]; then
	echo
	echo "ERROR : cannot find file $appName.tar.gz"
	echo "This installation package is incomplete or corrupted."
	echo "Installation aborted, $appNameFull was not installed."
  exit 1
fi

echo "$appName$appVersionFull installer log" >> $logFile
echo "________________________________________________________________________________"
echo " "
echo "  Welcome to the $appNameFull installer"
if [ $forceMode = "yes" ]; then
echo "  Force mode activated: no user interaction, all default choices will be used."
fi
echo " "

# Check if the script is being run as root or as a simple user.
if [ `id -u` -ne "0" ]
then
	echo "  You do not have root access."
	echo "  $appNameFull will be installed in your home directory by default."
	installType="user"
	appPrefixDefault=$appPrefixDefaultUser
	linkPrefixDefault=$linkPrefixDefaultUser
else
	echo "  You have root access."
	echo "  $appNameFull will be installed system wide by default"
	installType="root"
	appPrefixDefault=$appPrefixDefaultRoot
	linkPrefixDefault=$linkPrefixDefaultRoot
fi
echo "________________________________________________________________________________"

# Check system arch
vArch=`uname -m`
if [ $vArch = "x86_64" ] ; then
	echo
	echo "WARNING: This is a 64 bit system. This version of $appNameFull is a"
	echo "32 bit application. Although it may work on this system if the 32 bit"
	echo "system libraries are present, certain features may not work."
	echo "$appNameFull may need to interact with OpenOffice. This will fail if the"
	echo "two programs are compiled for different architectures."
	if [ $forceMode = "no" ]; then
		echo -n "Do you want to continue? [Y/n]: "
		read userRes
		userRes=`echo $userRes | tr 'A-Z' 'a-z'`
		if [ "$userRes" = "n" ]; then 
			echo "Installation aborted, $appNameFull was not installed."
	  	exit 1
		fi
	fi
	incorrectArch="true"
fi

# Display license.
if [ -f "license.app.txt" ] ; then 
	vLicFile="license.app.txt"
else
	vLicFile="license.scenari.txt"
fi
if [ -f "$vLicFile" ] && [ $forceMode = "no" ] ; then 
	echo
	echo "Please read and accept the following end user license before"
	echo "installing $appNameFull."
	echo -n "Press Enter to continue"
	read userRes
	more $vLicFile
	echo
	echo -n "Do you agree to the above license terms? [yes/NO] "
	read userRes
	userRes=`echo $userRes | tr 'A-Z' 'a-z'`
	if [ "$userRes" != "y" ] && [ "$userRes" != "yes" ] ; then 
		echo "If you don't agree to the license you can't install $appNameFull."
		echo "Installation aborted, $appNameFull was not installed."
	  exit 1
	fi
fi

appPathDefault=$appPrefixDefault"/"$appPublisher"/"$appName$appVersion
echo
if [ $forceMode = "no" ]; then
	# Prompt for install location.
	echo -n "Please give the installation location [$appPathDefault]: "
	read appPath
else
	echo "$appNameFull will be installed in $appPathDefault"
fi
if [ "$appPath" = "" ]; then
	appPath=$appPathDefault
fi

# Delete previous installation if folder exists
if [ -d $appPath ]; then
	if [ ! -w $appPath ]; then
		echo
		echo "You do not have write permission for $appPath."
		echo "Installation aborted, $appNameFull was not installed."
	  exit 1
	fi
	if [ $forceMode = "no" ]; then
		echo
		echo "WARNING: $appPath. is not empty."
		echo "This folder will be deleted before proceding."
		echo -n "Do you want to continue? [Y/n]: "
		read userRes
		userRes=`echo $userRes | tr 'A-Z' 'a-z'`
	else
		echo "WARNING: $appPath. is not empty."
		echo "This folder will be deleted and replaced with $appNameFull."
		userRes="y"
	fi
	if [ "$userRes" = "n" ]; then 
		echo "Installation aborted, $appNameFull was not installed."
  	exit 1
	fi
	echo -n "Delete previous installation..." >> $logFile
	rm -Rf $appPath >> $logFile 2>&1
	echo "OK" >> $logFile
	echo
fi

mkdir -p $appPath
if [ ! -w $appPath ]; then
	echo "You do not have write permission for $appPath."
	echo "Installation aborted, $appNameFull was not installed."
  exit 1
fi

#==============================================================================
# Unpack the application
#==============================================================================
#Unpack SCx to the right location in appPrefix
echo -n "Unpacking  $appNameFull..."
echo -n "Unpack $appNameFull to $appPath..." >> $logFile
mkdir -p $appPath >> $logFile 2>&1
tar -x -z --no-same-owner --no-same-permissions -f $appName".tar.gz"  -C $appPath >> $logFile 2>&1
#Unpack xulrunner...
tar -x -z --no-same-owner --no-same-permissions -f $appPath"/bin/xulrunner.tar.gz" -C $appPath"/bin/" >> $logFile 2>&1
rm -f $appPath"/bin/xulrunner.tar.gz" >> $logFile 2>&1
if [ -f $appPath"/bin/xulrunner/chrome.manifest.append" ]; then
	cat $appPath"/bin/xulrunner/chrome.manifest.append" >> $appPath"/bin/xulrunner/chrome.manifest"
	rm -f $appPath"/bin/xulrunner/chrome.manifest.append" >> $logFile 2>&1
fi
echo "OK"
echo "OK" >> $logFile
#Unpack and setup jre if it exists...
if [ -f $appPath"/bin/scServer/jre/jre.tar.gz" ]; then
	echo -n "Unpacking the embedded java runtime environment..."
	echo -n "Unpacking the embedded java runtime environment..." >> $logFile
	tar -x -z --no-same-owner --no-same-permissions -f $appPath"/bin/scServer/jre/jre.tar.gz" -C $appPath"/bin/scServer/jre/" >> $logFile 2>&1
	rm -f $appPath"/bin/scServer/jre/jre.tar.gz" >> $logFile 2>&1
	echo "OK"
	echo "OK" >> $logFile

	# Dump client classes...
	echo "Dump client classes:" >> $logFile
	$appPath/bin/scServer/jre/bin/java -client -Xshare:dump >> $logFile 2>&1
fi

#==============================================================================
# Setup the application
#==============================================================================
echo -n "Application setup..."

#Replace ==appPath== in launcher script
echo -n "Set up launcher script..." >> $logFile
appPathEsc=$(echo $appPath | sed -e 's/\//\\\//g')
sed -e "s/==app.path==/$appPathEsc/g" < $appPath"/"$appName > $appPath"/"$appName".tmp"
rm -f $appPath"/"$appName >> $logFile 2>&1
mv $appPath"/"$appName".tmp" $appPath"/"$appName >> $logFile 2>&1

#Replace ==app.nameFull== in launcher script
sed -e "s/==app.nameSig==/$appName$appVersion/g" < $appPath"/"$appName > $appPath"/"$appName".tmp"
rm -f $appPath"/"$appName >> $logFile 2>&1
mv $appPath"/"$appName".tmp" $appPath"/"$appName >> $logFile 2>&1
echo "OK" >> $logFile

#Set user rights
echo -n "Set user rights (chmod a+x) on various elements..." >> $logFile
chmod a+x $appPath"/"$appName >> $logFile 2>&1
if [ -f "$appPath/bin/scServer/nativlib/ffmpeg" ]; then
	chmod a+x $appPath"/bin/scServer/nativlib/ffmpeg" >> $logFile 2>&1
fi
if [ -f "$appPath/bin/scServer/nativlib/node" ]; then
	chmod a+x $appPath"/bin/scServer/nativlib/node" >> $logFile 2>&1
fi
if [ -f "$appPath/bin/scServer/nativlib/im/convert" ]; then
	chmod a+x $appPath"/bin/scServer/nativlib/im/convert" >> $logFile 2>&1
fi
echo "OK" >> $logFile

#Leave a trace of the the installed launcher's path for possible automated wsppack installs
echo -n "Leave a trace of the the installed launcher's path in $lastScAppBinPathFile..." >> $logFile
mkdir -p `dirname $lastScAppBinPathFile` >> /dev/null 2>&1
echo "vLastScAppBinPath=$appPath/$appName" > $lastScAppBinPathFile
echo "OK" >> $logFile

echo "OK"
echo "$appNameFull has been installed in $appPath"

#==============================================================================
# Setup the enviroment
#==============================================================================
# Enviroment integration (links menus etc)
echo "Enviroment integration:" >> $logFile
echo
if [ $forceMode = "no" ]; then
	echo -n "Do you want to incorprate $appNameFull into your enviroment? [Y/n]: "
	read userRes
	userRes=`echo $userRes | tr 'A-Z' 'a-z'`
else
	userRes="y"
fi
if [ "$userRes" = "" ] || [ "$userRes" = "y" ] ; then 
	# Add links
	if [ "$installType" = "root" ] && [ $forceMode = "no" ]; then
		echo -n "Please specify the path prefix for your system [$linkPrefixDefault]: "
		read linkPrefix
		if [ "$linkPrefix" = "" ] ; then
			linkPrefix=$linkPrefixDefault
		fi
	else
		linkPrefix=$linkPrefixDefault 
	fi
	if [ -d "$linkPrefix/bin" ]; then
		rm -f $linkPrefix"/bin/"$appName$appVersion >> $logFile 2>&1
		ln -s $appPath"/"$appName $linkPrefix"/bin/"$appName$appVersion >> $logFile 2>&1
		echo "The link $linkPrefix/bin/$appName$appVersion has been added"
		echo "The link $linkPrefix/bin/$appName$appVersion has been added" >> $logFile
	fi

	# Add menu item if possible...
	appDesktopFilePath="false"
	if [ "$installType" = "root" ] && [ -d $linkPrefix"/share/applications" ]; then
		appDesktopFilePath=$linkPrefix"/share/applications/"$appName$appVersion".desktop"
		echo "$appNameFull will be integated to your system."
	elif [ "$installType" = "user" ]; then
		if [ -d $HOME"/.local/share/applications" ]; then
			echo "$appNameFull will be integated to your system."
			appDesktopFilePath=$HOME"/.local/share/applications/"$appName$appVersion".desktop"
		elif [ -d "$HOME/Desktop" ]; then
			echo "$appNameFull will be added to your desktop."
			appDesktopFilePath=$HOME"/Desktop/"$appName$appVersion".desktop"
		fi
	fi
	if [ "$appDesktopFilePath" != "false" ]; then
		echo "[Desktop Entry]" > $appDesktopFilePath
		echo "Name=$appNameFull" >> $appDesktopFilePath
		echo "Encoding=UTF-8" >> $appDesktopFilePath
		echo "Exec=$appPath/$appName" >> $appDesktopFilePath
		echo "Terminal=false" >> $appDesktopFilePath
		echo "StartupNotify=true" >> $appDesktopFilePath
		if [ -f "$appPath/bin/$appClientName/chrome/icons/default/scenari.svg" ]; then
			echo "Icon=$appPath/bin/$appClientName/chrome/icons/default/scenari.svg" >> $appDesktopFilePath
		else
			echo "Icon=$appPath/bin/$appClientName/chrome/icons/default/scenari.png" >> $appDesktopFilePath
		fi
		echo "Type=Application" >> $appDesktopFilePath
		echo "Categories=Office;" >> $appDesktopFilePath
		echo "Comment=$appDescription" >> $appDesktopFilePath
		chmod 755 $appDesktopFilePath  >> $logFile 2>&1
		echo "The file $appDesktopFilePath has been created" >> $logFile
	fi
else
	echo "$appNameFull has not been incorporated into your enviroment"
fi
echo
echo "$appNameFull installation finished"
echo "$appNameFull installation finished" >> $logFile
echo "A log of this installation is stored in :"
echo "$logFile"

chmod 666 $logFile >> /dev/null 2>&1
